import mongoengine as mongo
from omni.pro import redis
from omni.pro.config import Config
from omni.pro.logger import configure_logger

logger = configure_logger(name=__name__)


def register_logger_connection(tenant, manager: redis.RedisManager):
    conn = manager.get_mongodb_config(Config.LOGGER_ID, tenant)
    mongo.register_connection(
        alias=f"{tenant}_{conn['name']}",
        name=conn["name"],
        host=conn["host"],
        port=int(conn["port"]),
        username=conn["user"],
        password=conn["password"],
        **conn["complement"],
    )


def ms_register_connection(logger_oms=False):
    manager = redis.get_redis_manager()
    tenants = manager.get_tenant_codes()
    # Registrar todas las conexiones
    for idx, tenant in enumerate(tenants):
        logger.info(f"Register connection for tenant {tenant}")
        conn = manager.get_mongodb_config(Config.SERVICE_ID, tenant)
        eval_conn = conn.copy()
        if Config.DEBUG:
            del eval_conn["complement"]
        if not all(eval_conn.values()):
            continue

        mongo.register_connection(
            alias=f"{tenant}_{conn['name']}",
            name=conn["name"],
            host=conn["host"],
            port=int(conn["port"]),
            username=conn["user"],
            password=conn["password"],
            **conn["complement"],
        )

        if logger_oms:
            register_logger_connection(tenant, manager)

        if idx == 0:
            mongo.register_connection(
                alias=mongo.DEFAULT_CONNECTION_NAME,
                name=mongo.DEFAULT_CONNECTION_NAME,
                host=conn["host"],
                port=int(conn["port"]),
                username=conn["user"],
                password=conn["password"],
                **conn["complement"],
            )



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
from omni.pro.protos.v1.rules import appointment_template_line_pb2 as v1_dot_rules_dot_appointment__template__line__pb2


class AppointmentTemplateLineServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.AppointmentTemplateLineCreate = channel.unary_unary(
            "/pro.omni.oms.api.v1.rules.appointment_template_line.AppointmentTemplateLineService/AppointmentTemplateLineCreate",
            request_serializer=v1_dot_rules_dot_appointment__template__line__pb2.AppointmentTemplateLineCreateRequest.SerializeToString,
            response_deserializer=v1_dot_rules_dot_appointment__template__line__pb2.AppointmentTemplateLineCreateResponse.FromString,
        )
        self.AppointmentTemplateLineRead = channel.unary_unary(
            "/pro.omni.oms.api.v1.rules.appointment_template_line.AppointmentTemplateLineService/AppointmentTemplateLineRead",
            request_serializer=v1_dot_rules_dot_appointment__template__line__pb2.AppointmentTemplateLineReadRequest.SerializeToString,
            response_deserializer=v1_dot_rules_dot_appointment__template__line__pb2.AppointmentTemplateLineReadResponse.FromString,
        )
        self.AppointmentTemplateLineUpdate = channel.unary_unary(
            "/pro.omni.oms.api.v1.rules.appointment_template_line.AppointmentTemplateLineService/AppointmentTemplateLineUpdate",
            request_serializer=v1_dot_rules_dot_appointment__template__line__pb2.AppointmentTemplateLineUpdateRequest.SerializeToString,
            response_deserializer=v1_dot_rules_dot_appointment__template__line__pb2.AppointmentTemplateLineUpdateResponse.FromString,
        )
        self.AppointmentTemplateLineDelete = channel.unary_unary(
            "/pro.omni.oms.api.v1.rules.appointment_template_line.AppointmentTemplateLineService/AppointmentTemplateLineDelete",
            request_serializer=v1_dot_rules_dot_appointment__template__line__pb2.AppointmentTemplateLineDeleteRequest.SerializeToString,
            response_deserializer=v1_dot_rules_dot_appointment__template__line__pb2.AppointmentTemplateLineDeleteResponse.FromString,
        )


class AppointmentTemplateLineServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def AppointmentTemplateLineCreate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def AppointmentTemplateLineRead(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def AppointmentTemplateLineUpdate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def AppointmentTemplateLineDelete(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_AppointmentTemplateLineServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "AppointmentTemplateLineCreate": grpc.unary_unary_rpc_method_handler(
            servicer.AppointmentTemplateLineCreate,
            request_deserializer=v1_dot_rules_dot_appointment__template__line__pb2.AppointmentTemplateLineCreateRequest.FromString,
            response_serializer=v1_dot_rules_dot_appointment__template__line__pb2.AppointmentTemplateLineCreateResponse.SerializeToString,
        ),
        "AppointmentTemplateLineRead": grpc.unary_unary_rpc_method_handler(
            servicer.AppointmentTemplateLineRead,
            request_deserializer=v1_dot_rules_dot_appointment__template__line__pb2.AppointmentTemplateLineReadRequest.FromString,
            response_serializer=v1_dot_rules_dot_appointment__template__line__pb2.AppointmentTemplateLineReadResponse.SerializeToString,
        ),
        "AppointmentTemplateLineUpdate": grpc.unary_unary_rpc_method_handler(
            servicer.AppointmentTemplateLineUpdate,
            request_deserializer=v1_dot_rules_dot_appointment__template__line__pb2.AppointmentTemplateLineUpdateRequest.FromString,
            response_serializer=v1_dot_rules_dot_appointment__template__line__pb2.AppointmentTemplateLineUpdateResponse.SerializeToString,
        ),
        "AppointmentTemplateLineDelete": grpc.unary_unary_rpc_method_handler(
            servicer.AppointmentTemplateLineDelete,
            request_deserializer=v1_dot_rules_dot_appointment__template__line__pb2.AppointmentTemplateLineDeleteRequest.FromString,
            response_serializer=v1_dot_rules_dot_appointment__template__line__pb2.AppointmentTemplateLineDeleteResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "pro.omni.oms.api.v1.rules.appointment_template_line.AppointmentTemplateLineService", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class AppointmentTemplateLineService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def AppointmentTemplateLineCreate(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.rules.appointment_template_line.AppointmentTemplateLineService/AppointmentTemplateLineCreate",
            v1_dot_rules_dot_appointment__template__line__pb2.AppointmentTemplateLineCreateRequest.SerializeToString,
            v1_dot_rules_dot_appointment__template__line__pb2.AppointmentTemplateLineCreateResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def AppointmentTemplateLineRead(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.rules.appointment_template_line.AppointmentTemplateLineService/AppointmentTemplateLineRead",
            v1_dot_rules_dot_appointment__template__line__pb2.AppointmentTemplateLineReadRequest.SerializeToString,
            v1_dot_rules_dot_appointment__template__line__pb2.AppointmentTemplateLineReadResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def AppointmentTemplateLineUpdate(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.rules.appointment_template_line.AppointmentTemplateLineService/AppointmentTemplateLineUpdate",
            v1_dot_rules_dot_appointment__template__line__pb2.AppointmentTemplateLineUpdateRequest.SerializeToString,
            v1_dot_rules_dot_appointment__template__line__pb2.AppointmentTemplateLineUpdateResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def AppointmentTemplateLineDelete(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.rules.appointment_template_line.AppointmentTemplateLineService/AppointmentTemplateLineDelete",
            v1_dot_rules_dot_appointment__template__line__pb2.AppointmentTemplateLineDeleteRequest.SerializeToString,
            v1_dot_rules_dot_appointment__template__line__pb2.AppointmentTemplateLineDeleteResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )



from typing import ClassVar as _ClassVar
from typing import Iterable as _Iterable
from typing import Mapping as _Mapping
from typing import Optional as _Optional
from typing import Union as _Union

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import struct_pb2 as _struct_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf import wrappers_pb2 as _wrappers_pb2
from google.protobuf.internal import containers as _containers
from omni.pro.protos.common import base_pb2 as _base_pb2
from omni.pro.protos.v1.stock import address_pb2 as _address_pb2
from omni.pro.protos.v1.stock import attachment_pb2 as _attachment_pb2
from omni.pro.protos.v1.stock import carrier_pb2 as _carrier_pb2
from omni.pro.protos.v1.stock import client_pb2 as _client_pb2
from omni.pro.protos.v1.stock import delivery_method_pb2 as _delivery_method_pb2
from omni.pro.protos.v1.stock import location_pb2 as _location_pb2
from omni.pro.protos.v1.stock import order_pb2 as _order_pb2
from omni.pro.protos.v1.stock import payment_method_pb2 as _payment_method_pb2
from omni.pro.protos.v1.stock import picking_type_pb2 as _picking_type_pb2
from omni.pro.protos.v1.stock import procurement_group_pb2 as _procurement_group_pb2
from omni.pro.protos.v1.stock import sale_pb2 as _sale_pb2
from omni.pro.protos.v1.stock import user_pb2 as _user_pb2

DESCRIPTOR: _descriptor.FileDescriptor

class Picking(_message.Message):
    __slots__ = [
        "id",
        "name",
        "picking_type",
        "location",
        "location_dest",
        "attachment_guide",
        "attachment_invoice",
        "origin",
        "date_start_preparation",
        "date_done",
        "scheduled_date",
        "time_total_preparation",
        "time_assigned",
        "carrier",
        "date_delivery",
        "carrier_tracking_ref",
        "group",
        "weight",
        "shipping_weight",
        "state",
        "dependency",
        "note",
        "carrier_tracking_url",
        "shipping_receives",
        "date_validated",
        "sale_date_order",
        "client",
        "delivery_method",
        "order",
        "payment_method",
        "sale",
        "user",
        "shipping_address",
        "active",
        "warehouse",
        "stock_moves",
        "stock_move_lines",
        "external_id",
        "object_audit",
    ]
    ID_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    PICKING_TYPE_FIELD_NUMBER: _ClassVar[int]
    LOCATION_FIELD_NUMBER: _ClassVar[int]
    LOCATION_DEST_FIELD_NUMBER: _ClassVar[int]
    ATTACHMENT_GUIDE_FIELD_NUMBER: _ClassVar[int]
    ATTACHMENT_INVOICE_FIELD_NUMBER: _ClassVar[int]
    ORIGIN_FIELD_NUMBER: _ClassVar[int]
    DATE_START_PREPARATION_FIELD_NUMBER: _ClassVar[int]
    DATE_DONE_FIELD_NUMBER: _ClassVar[int]
    SCHEDULED_DATE_FIELD_NUMBER: _ClassVar[int]
    TIME_TOTAL_PREPARATION_FIELD_NUMBER: _ClassVar[int]
    TIME_ASSIGNED_FIELD_NUMBER: _ClassVar[int]
    CARRIER_FIELD_NUMBER: _ClassVar[int]
    DATE_DELIVERY_FIELD_NUMBER: _ClassVar[int]
    CARRIER_TRACKING_REF_FIELD_NUMBER: _ClassVar[int]
    GROUP_FIELD_NUMBER: _ClassVar[int]
    WEIGHT_FIELD_NUMBER: _ClassVar[int]
    SHIPPING_WEIGHT_FIELD_NUMBER: _ClassVar[int]
    STATE_FIELD_NUMBER: _ClassVar[int]
    DEPENDENCY_FIELD_NUMBER: _ClassVar[int]
    NOTE_FIELD_NUMBER: _ClassVar[int]
    CARRIER_TRACKING_URL_FIELD_NUMBER: _ClassVar[int]
    SHIPPING_RECEIVES_FIELD_NUMBER: _ClassVar[int]
    DATE_VALIDATED_FIELD_NUMBER: _ClassVar[int]
    SALE_DATE_ORDER_FIELD_NUMBER: _ClassVar[int]
    CLIENT_FIELD_NUMBER: _ClassVar[int]
    DELIVERY_METHOD_FIELD_NUMBER: _ClassVar[int]
    ORDER_FIELD_NUMBER: _ClassVar[int]
    PAYMENT_METHOD_FIELD_NUMBER: _ClassVar[int]
    SALE_FIELD_NUMBER: _ClassVar[int]
    USER_FIELD_NUMBER: _ClassVar[int]
    SHIPPING_ADDRESS_FIELD_NUMBER: _ClassVar[int]
    ACTIVE_FIELD_NUMBER: _ClassVar[int]
    WAREHOUSE_FIELD_NUMBER: _ClassVar[int]
    STOCK_MOVES_FIELD_NUMBER: _ClassVar[int]
    STOCK_MOVE_LINES_FIELD_NUMBER: _ClassVar[int]
    EXTERNAL_ID_FIELD_NUMBER: _ClassVar[int]
    OBJECT_AUDIT_FIELD_NUMBER: _ClassVar[int]
    id: int
    name: str
    picking_type: _picking_type_pb2.PickingType
    location: _location_pb2.Location
    location_dest: _location_pb2.Location
    attachment_guide: _attachment_pb2.Attachment
    attachment_invoice: _attachment_pb2.Attachment
    origin: str
    date_start_preparation: _timestamp_pb2.Timestamp
    date_done: _timestamp_pb2.Timestamp
    scheduled_date: _timestamp_pb2.Timestamp
    time_total_preparation: float
    time_assigned: float
    carrier: _carrier_pb2.Carrier
    date_delivery: _timestamp_pb2.Timestamp
    carrier_tracking_ref: str
    group: _procurement_group_pb2.ProcurementGroup
    weight: float
    shipping_weight: float
    state: str
    dependency: _base_pb2.ObjectResponse
    note: str
    carrier_tracking_url: str
    shipping_receives: _struct_pb2.Struct
    date_validated: _timestamp_pb2.Timestamp
    sale_date_order: _timestamp_pb2.Timestamp
    client: _client_pb2.Client
    delivery_method: _delivery_method_pb2.DeliveryMethod
    order: _order_pb2.Order
    payment_method: _payment_method_pb2.PaymentMethod
    sale: _sale_pb2.Sale
    user: _user_pb2.User
    shipping_address: _address_pb2.Address
    active: _wrappers_pb2.BoolValue
    warehouse: _struct_pb2.Struct
    stock_moves: _containers.RepeatedCompositeFieldContainer[_struct_pb2.Struct]
    stock_move_lines: _containers.RepeatedCompositeFieldContainer[_struct_pb2.Struct]
    external_id: str
    object_audit: _base_pb2.ObjectAudit
    def __init__(
        self,
        id: _Optional[int] = ...,
        name: _Optional[str] = ...,
        picking_type: _Optional[_Union[_picking_type_pb2.PickingType, _Mapping]] = ...,
        location: _Optional[_Union[_location_pb2.Location, _Mapping]] = ...,
        location_dest: _Optional[_Union[_location_pb2.Location, _Mapping]] = ...,
        attachment_guide: _Optional[_Union[_attachment_pb2.Attachment, _Mapping]] = ...,
        attachment_invoice: _Optional[_Union[_attachment_pb2.Attachment, _Mapping]] = ...,
        origin: _Optional[str] = ...,
        date_start_preparation: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...,
        date_done: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...,
        scheduled_date: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...,
        time_total_preparation: _Optional[float] = ...,
        time_assigned: _Optional[float] = ...,
        carrier: _Optional[_Union[_carrier_pb2.Carrier, _Mapping]] = ...,
        date_delivery: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...,
        carrier_tracking_ref: _Optional[str] = ...,
        group: _Optional[_Union[_procurement_group_pb2.ProcurementGroup, _Mapping]] = ...,
        weight: _Optional[float] = ...,
        shipping_weight: _Optional[float] = ...,
        state: _Optional[str] = ...,
        dependency: _Optional[_Union[_base_pb2.ObjectResponse, _Mapping]] = ...,
        note: _Optional[str] = ...,
        carrier_tracking_url: _Optional[str] = ...,
        shipping_receives: _Optional[_Union[_struct_pb2.Struct, _Mapping]] = ...,
        date_validated: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...,
        sale_date_order: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...,
        client: _Optional[_Union[_client_pb2.Client, _Mapping]] = ...,
        delivery_method: _Optional[_Union[_delivery_method_pb2.DeliveryMethod, _Mapping]] = ...,
        order: _Optional[_Union[_order_pb2.Order, _Mapping]] = ...,
        payment_method: _Optional[_Union[_payment_method_pb2.PaymentMethod, _Mapping]] = ...,
        sale: _Optional[_Union[_sale_pb2.Sale, _Mapping]] = ...,
        user: _Optional[_Union[_user_pb2.User, _Mapping]] = ...,
        shipping_address: _Optional[_Union[_address_pb2.Address, _Mapping]] = ...,
        active: _Optional[_Union[_wrappers_pb2.BoolValue, _Mapping]] = ...,
        warehouse: _Optional[_Union[_struct_pb2.Struct, _Mapping]] = ...,
        stock_moves: _Optional[_Iterable[_Union[_struct_pb2.Struct, _Mapping]]] = ...,
        stock_move_lines: _Optional[_Iterable[_Union[_struct_pb2.Struct, _Mapping]]] = ...,
        external_id: _Optional[str] = ...,
        object_audit: _Optional[_Union[_base_pb2.ObjectAudit, _Mapping]] = ...,
    ) -> None: ...

class PickingCreateRequest(_message.Message):
    __slots__ = [
        "name",
        "picking_type_id",
        "location_id",
        "location_dest_id",
        "attachment_guide_id",
        "attachment_invoice_id",
        "origin",
        "date_start_preparation",
        "date_done",
        "scheduled_date",
        "time_total_preparation",
        "time_assigned",
        "carrier_id",
        "date_delivery",
        "carrier_tracking_ref",
        "group_id",
        "weight",
        "shipping_weight",
        "state",
        "dependency_id",
        "note",
        "carrier_tracking_url",
        "date_validated",
        "sale_date_order",
        "client_id",
        "delivery_method_id",
        "order_id",
        "payment_method_id",
        "sale_id",
        "user_id",
        "shipping_address_code",
        "warehouse_id",
        "shipping_receives",
        "external_id",
        "context",
    ]
    NAME_FIELD_NUMBER: _ClassVar[int]
    PICKING_TYPE_ID_FIELD_NUMBER: _ClassVar[int]
    LOCATION_ID_FIELD_NUMBER: _ClassVar[int]
    LOCATION_DEST_ID_FIELD_NUMBER: _ClassVar[int]
    ATTACHMENT_GUIDE_ID_FIELD_NUMBER: _ClassVar[int]
    ATTACHMENT_INVOICE_ID_FIELD_NUMBER: _ClassVar[int]
    ORIGIN_FIELD_NUMBER: _ClassVar[int]
    DATE_START_PREPARATION_FIELD_NUMBER: _ClassVar[int]
    DATE_DONE_FIELD_NUMBER: _ClassVar[int]
    SCHEDULED_DATE_FIELD_NUMBER: _ClassVar[int]
    TIME_TOTAL_PREPARATION_FIELD_NUMBER: _ClassVar[int]
    TIME_ASSIGNED_FIELD_NUMBER: _ClassVar[int]
    CARRIER_ID_FIELD_NUMBER: _ClassVar[int]
    DATE_DELIVERY_FIELD_NUMBER: _ClassVar[int]
    CARRIER_TRACKING_REF_FIELD_NUMBER: _ClassVar[int]
    GROUP_ID_FIELD_NUMBER: _ClassVar[int]
    WEIGHT_FIELD_NUMBER: _ClassVar[int]
    SHIPPING_WEIGHT_FIELD_NUMBER: _ClassVar[int]
    STATE_FIELD_NUMBER: _ClassVar[int]
    DEPENDENCY_ID_FIELD_NUMBER: _ClassVar[int]
    NOTE_FIELD_NUMBER: _ClassVar[int]
    CARRIER_TRACKING_URL_FIELD_NUMBER: _ClassVar[int]
    DATE_VALIDATED_FIELD_NUMBER: _ClassVar[int]
    SALE_DATE_ORDER_FIELD_NUMBER: _ClassVar[int]
    CLIENT_ID_FIELD_NUMBER: _ClassVar[int]
    DELIVERY_METHOD_ID_FIELD_NUMBER: _ClassVar[int]
    ORDER_ID_FIELD_NUMBER: _ClassVar[int]
    PAYMENT_METHOD_ID_FIELD_NUMBER: _ClassVar[int]
    SALE_ID_FIELD_NUMBER: _ClassVar[int]
    USER_ID_FIELD_NUMBER: _ClassVar[int]
    SHIPPING_ADDRESS_CODE_FIELD_NUMBER: _ClassVar[int]
    WAREHOUSE_ID_FIELD_NUMBER: _ClassVar[int]
    SHIPPING_RECEIVES_FIELD_NUMBER: _ClassVar[int]
    EXTERNAL_ID_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    name: str
    picking_type_id: int
    location_id: int
    location_dest_id: int
    attachment_guide_id: int
    attachment_invoice_id: int
    origin: str
    date_start_preparation: _timestamp_pb2.Timestamp
    date_done: _timestamp_pb2.Timestamp
    scheduled_date: _timestamp_pb2.Timestamp
    time_total_preparation: float
    time_assigned: float
    carrier_id: int
    date_delivery: _timestamp_pb2.Timestamp
    carrier_tracking_ref: str
    group_id: int
    weight: float
    shipping_weight: float
    state: str
    dependency_id: int
    note: str
    carrier_tracking_url: str
    date_validated: _timestamp_pb2.Timestamp
    sale_date_order: _timestamp_pb2.Timestamp
    client_id: str
    delivery_method_id: str
    order_id: int
    payment_method_id: str
    sale_id: int
    user_id: str
    shipping_address_code: str
    warehouse_id: int
    shipping_receives: _struct_pb2.Struct
    external_id: str
    context: _base_pb2.Context
    def __init__(
        self,
        name: _Optional[str] = ...,
        picking_type_id: _Optional[int] = ...,
        location_id: _Optional[int] = ...,
        location_dest_id: _Optional[int] = ...,
        attachment_guide_id: _Optional[int] = ...,
        attachment_invoice_id: _Optional[int] = ...,
        origin: _Optional[str] = ...,
        date_start_preparation: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...,
        date_done: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...,
        scheduled_date: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...,
        time_total_preparation: _Optional[float] = ...,
        time_assigned: _Optional[float] = ...,
        carrier_id: _Optional[int] = ...,
        date_delivery: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...,
        carrier_tracking_ref: _Optional[str] = ...,
        group_id: _Optional[int] = ...,
        weight: _Optional[float] = ...,
        shipping_weight: _Optional[float] = ...,
        state: _Optional[str] = ...,
        dependency_id: _Optional[int] = ...,
        note: _Optional[str] = ...,
        carrier_tracking_url: _Optional[str] = ...,
        date_validated: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...,
        sale_date_order: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...,
        client_id: _Optional[str] = ...,
        delivery_method_id: _Optional[str] = ...,
        order_id: _Optional[int] = ...,
        payment_method_id: _Optional[str] = ...,
        sale_id: _Optional[int] = ...,
        user_id: _Optional[str] = ...,
        shipping_address_code: _Optional[str] = ...,
        warehouse_id: _Optional[int] = ...,
        shipping_receives: _Optional[_Union[_struct_pb2.Struct, _Mapping]] = ...,
        external_id: _Optional[str] = ...,
        context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...,
    ) -> None: ...

class PickingCreateResponse(_message.Message):
    __slots__ = ["response_standard", "picking"]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    PICKING_FIELD_NUMBER: _ClassVar[int]
    response_standard: _base_pb2.ResponseStandard
    picking: Picking
    def __init__(
        self,
        response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...,
        picking: _Optional[_Union[Picking, _Mapping]] = ...,
    ) -> None: ...

class PickingReadRequest(_message.Message):
    __slots__ = ["group_by", "sort_by", "fields", "filter", "paginated", "id", "context"]
    GROUP_BY_FIELD_NUMBER: _ClassVar[int]
    SORT_BY_FIELD_NUMBER: _ClassVar[int]
    FIELDS_FIELD_NUMBER: _ClassVar[int]
    FILTER_FIELD_NUMBER: _ClassVar[int]
    PAGINATED_FIELD_NUMBER: _ClassVar[int]
    ID_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    group_by: _containers.RepeatedCompositeFieldContainer[_base_pb2.GroupBy]
    sort_by: _base_pb2.SortBy
    fields: _base_pb2.Fields
    filter: _base_pb2.Filter
    paginated: _base_pb2.Paginated
    id: int
    context: _base_pb2.Context
    def __init__(
        self,
        group_by: _Optional[_Iterable[_Union[_base_pb2.GroupBy, _Mapping]]] = ...,
        sort_by: _Optional[_Union[_base_pb2.SortBy, _Mapping]] = ...,
        fields: _Optional[_Union[_base_pb2.Fields, _Mapping]] = ...,
        filter: _Optional[_Union[_base_pb2.Filter, _Mapping]] = ...,
        paginated: _Optional[_Union[_base_pb2.Paginated, _Mapping]] = ...,
        id: _Optional[int] = ...,
        context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...,
    ) -> None: ...

class PickingReadResponse(_message.Message):
    __slots__ = ["response_standard", "meta_data", "pickings"]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    META_DATA_FIELD_NUMBER: _ClassVar[int]
    PICKINGS_FIELD_NUMBER: _ClassVar[int]
    response_standard: _base_pb2.ResponseStandard
    meta_data: _base_pb2.MetaData
    pickings: _containers.RepeatedCompositeFieldContainer[Picking]
    def __init__(
        self,
        response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...,
        meta_data: _Optional[_Union[_base_pb2.MetaData, _Mapping]] = ...,
        pickings: _Optional[_Iterable[_Union[Picking, _Mapping]]] = ...,
    ) -> None: ...

class PickingUpdateRequest(_message.Message):
    __slots__ = ["picking", "context"]
    PICKING_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    picking: Picking
    context: _base_pb2.Context
    def __init__(
        self,
        picking: _Optional[_Union[Picking, _Mapping]] = ...,
        context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...,
    ) -> None: ...

class PickingUpdateResponse(_message.Message):
    __slots__ = ["response_standard", "picking"]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    PICKING_FIELD_NUMBER: _ClassVar[int]
    response_standard: _base_pb2.ResponseStandard
    picking: Picking
    def __init__(
        self,
        response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...,
        picking: _Optional[_Union[Picking, _Mapping]] = ...,
    ) -> None: ...

class PickingDeleteRequest(_message.Message):
    __slots__ = ["id", "context"]
    ID_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    id: int
    context: _base_pb2.Context
    def __init__(
        self, id: _Optional[int] = ..., context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...
    ) -> None: ...

class PickingDeleteResponse(_message.Message):
    __slots__ = ["response_standard"]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    response_standard: _base_pb2.ResponseStandard
    def __init__(self, response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...) -> None: ...

class ValidatePickingRequest(_message.Message):
    __slots__ = ["id", "context"]
    ID_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    id: int
    context: _base_pb2.Context
    def __init__(
        self, id: _Optional[int] = ..., context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...
    ) -> None: ...

class ValidatePickingResponse(_message.Message):
    __slots__ = ["response_standard", "data"]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    DATA_FIELD_NUMBER: _ClassVar[int]
    response_standard: _base_pb2.ResponseStandard
    data: _struct_pb2.Struct
    def __init__(
        self,
        response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...,
        data: _Optional[_Union[_struct_pb2.Struct, _Mapping]] = ...,
    ) -> None: ...

class PickingMovesRequest(_message.Message):
    __slots__ = ["id", "context"]
    ID_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    id: int
    context: _base_pb2.Context
    def __init__(
        self, id: _Optional[int] = ..., context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...
    ) -> None: ...

class PickingMovesResponse(_message.Message):
    __slots__ = ["response_standard", "data"]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    DATA_FIELD_NUMBER: _ClassVar[int]
    response_standard: _base_pb2.ResponseStandard
    data: _struct_pb2.Struct
    def __init__(
        self,
        response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...,
        data: _Optional[_Union[_struct_pb2.Struct, _Mapping]] = ...,
    ) -> None: ...

class OrderConfirmRequest(_message.Message):
    __slots__ = ["order_data", "context"]
    ORDER_DATA_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    order_data: _struct_pb2.Struct
    context: _base_pb2.Context
    def __init__(
        self,
        order_data: _Optional[_Union[_struct_pb2.Struct, _Mapping]] = ...,
        context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...,
    ) -> None: ...

class OrderConfirmResponse(_message.Message):
    __slots__ = ["response_standard", "pickings"]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    PICKINGS_FIELD_NUMBER: _ClassVar[int]
    response_standard: _base_pb2.ResponseStandard
    pickings: _containers.RepeatedCompositeFieldContainer[Picking]
    def __init__(
        self,
        response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...,
        pickings: _Optional[_Iterable[_Union[Picking, _Mapping]]] = ...,
    ) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: v1/rules/appointment_line.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from omni.pro.protos.common import base_pb2 as common_dot_base__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x1fv1/rules/appointment_line.proto\x12*pro.omni.oms.api.v1.rules.appointment_line\x1a\x11\x63ommon/base.proto\x1a\x1egoogle/protobuf/wrappers.proto"\x89\x02\n\x0f\x41ppointmentLine\x12\n\n\x02id\x18\x01 \x01(\t\x12\x12\n\nhour_start\x18\x02 \x01(\t\x12\x10\n\x08hour_end\x18\x03 \x01(\t\x12\x15\n\rorder_numbers\x18\x04 \x01(\x05\x12\x1d\n\x15\x61ppointment_available\x18\x05 \x01(\x05\x12\x0c\n\x04\x63ids\x18\x06 \x03(\t\x12*\n\x06\x61\x63tive\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x12\x13\n\x0b\x65xternal_id\x18\x08 \x01(\t\x12?\n\x0cobject_audit\x18\t \x01(\x0b\x32).pro.omni.oms.api.common.base.ObjectAudit"\xd5\x01\n\x1c\x41ppointmentLineCreateRequest\x12\x12\n\nhour_start\x18\x01 \x01(\t\x12\x10\n\x08hour_end\x18\x02 \x01(\t\x12\x15\n\rorder_numbers\x18\x03 \x01(\x05\x12\x1d\n\x15\x61ppointment_available\x18\x04 \x01(\x05\x12\x0c\n\x04\x63ids\x18\x05 \x03(\t\x12\x13\n\x0b\x65xternal_id\x18\x06 \x01(\t\x12\x36\n\x07\x63ontext\x18\x07 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xc1\x01\n\x1d\x41ppointmentLineCreateResponse\x12U\n\x10\x61ppointment_line\x18\x01 \x01(\x0b\x32;.pro.omni.oms.api.v1.rules.appointment_line.AppointmentLine\x12I\n\x11response_standard\x18\x02 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"\xf8\x02\n\x1a\x41ppointmentLineReadRequest\x12\x37\n\x08group_by\x18\x01 \x03(\x0b\x32%.pro.omni.oms.api.common.base.GroupBy\x12\x35\n\x07sort_by\x18\x02 \x01(\x0b\x32$.pro.omni.oms.api.common.base.SortBy\x12\x34\n\x06\x66ields\x18\x03 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Fields\x12\x34\n\x06\x66ilter\x18\x04 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Filter\x12:\n\tpaginated\x18\x05 \x01(\x0b\x32\'.pro.omni.oms.api.common.base.Paginated\x12\n\n\x02id\x18\x06 \x01(\t\x12\x36\n\x07\x63ontext\x18\x07 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xfb\x01\n\x1b\x41ppointmentLineReadResponse\x12V\n\x11\x61ppointment_lines\x18\x01 \x03(\x0b\x32;.pro.omni.oms.api.v1.rules.appointment_line.AppointmentLine\x12\x39\n\tmeta_data\x18\x02 \x01(\x0b\x32&.pro.omni.oms.api.common.base.MetaData\x12I\n\x11response_standard\x18\x03 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"\xad\x01\n\x1c\x41ppointmentLineUpdateRequest\x12U\n\x10\x61ppointment_line\x18\x01 \x01(\x0b\x32;.pro.omni.oms.api.v1.rules.appointment_line.AppointmentLine\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xc1\x01\n\x1d\x41ppointmentLineUpdateResponse\x12U\n\x10\x61ppointment_line\x18\x01 \x01(\x0b\x32;.pro.omni.oms.api.v1.rules.appointment_line.AppointmentLine\x12I\n\x11response_standard\x18\x02 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"b\n\x1c\x41ppointmentLineDeleteRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"j\n\x1d\x41ppointmentLineDeleteResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard2\xd6\x05\n\x16\x41ppointmentLineService\x12\xae\x01\n\x15\x41ppointmentLineCreate\x12H.pro.omni.oms.api.v1.rules.appointment_line.AppointmentLineCreateRequest\x1aI.pro.omni.oms.api.v1.rules.appointment_line.AppointmentLineCreateResponse"\x00\x12\xa8\x01\n\x13\x41ppointmentLineRead\x12\x46.pro.omni.oms.api.v1.rules.appointment_line.AppointmentLineReadRequest\x1aG.pro.omni.oms.api.v1.rules.appointment_line.AppointmentLineReadResponse"\x00\x12\xae\x01\n\x15\x41ppointmentLineUpdate\x12H.pro.omni.oms.api.v1.rules.appointment_line.AppointmentLineUpdateRequest\x1aI.pro.omni.oms.api.v1.rules.appointment_line.AppointmentLineUpdateResponse"\x00\x12\xae\x01\n\x15\x41ppointmentLineDelete\x12H.pro.omni.oms.api.v1.rules.appointment_line.AppointmentLineDeleteRequest\x1aI.pro.omni.oms.api.v1.rules.appointment_line.AppointmentLineDeleteResponse"\x00\x62\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "v1.rules.appointment_line_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    _globals["_APPOINTMENTLINE"]._serialized_start = 131
    _globals["_APPOINTMENTLINE"]._serialized_end = 396
    _globals["_APPOINTMENTLINECREATEREQUEST"]._serialized_start = 399
    _globals["_APPOINTMENTLINECREATEREQUEST"]._serialized_end = 612
    _globals["_APPOINTMENTLINECREATERESPONSE"]._serialized_start = 615
    _globals["_APPOINTMENTLINECREATERESPONSE"]._serialized_end = 808
    _globals["_APPOINTMENTLINEREADREQUEST"]._serialized_start = 811
    _globals["_APPOINTMENTLINEREADREQUEST"]._serialized_end = 1187
    _globals["_APPOINTMENTLINEREADRESPONSE"]._serialized_start = 1190
    _globals["_APPOINTMENTLINEREADRESPONSE"]._serialized_end = 1441
    _globals["_APPOINTMENTLINEUPDATEREQUEST"]._serialized_start = 1444
    _globals["_APPOINTMENTLINEUPDATEREQUEST"]._serialized_end = 1617
    _globals["_APPOINTMENTLINEUPDATERESPONSE"]._serialized_start = 1620
    _globals["_APPOINTMENTLINEUPDATERESPONSE"]._serialized_end = 1813
    _globals["_APPOINTMENTLINEDELETEREQUEST"]._serialized_start = 1815
    _globals["_APPOINTMENTLINEDELETEREQUEST"]._serialized_end = 1913
    _globals["_APPOINTMENTLINEDELETERESPONSE"]._serialized_start = 1915
    _globals["_APPOINTMENTLINEDELETERESPONSE"]._serialized_end = 2021
    _globals["_APPOINTMENTLINESERVICE"]._serialized_start = 2024
    _globals["_APPOINTMENTLINESERVICE"]._serialized_end = 2750
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
from omni.pro.protos.v1.users import notification_pb2 as v1_dot_users_dot_notification__pb2


class NotificationServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.NotificationCreate = channel.unary_unary(
            "/pro.omni.oms.api.v1.users.notification.NotificationService/NotificationCreate",
            request_serializer=v1_dot_users_dot_notification__pb2.NotificationCreateRequest.SerializeToString,
            response_deserializer=v1_dot_users_dot_notification__pb2.NotificationCreateResponse.FromString,
        )
        self.NotificationRead = channel.unary_unary(
            "/pro.omni.oms.api.v1.users.notification.NotificationService/NotificationRead",
            request_serializer=v1_dot_users_dot_notification__pb2.NotificationReadRequest.SerializeToString,
            response_deserializer=v1_dot_users_dot_notification__pb2.NotificationReadResponse.FromString,
        )
        self.NotificationUpdate = channel.unary_unary(
            "/pro.omni.oms.api.v1.users.notification.NotificationService/NotificationUpdate",
            request_serializer=v1_dot_users_dot_notification__pb2.NotificationUpdateRequest.SerializeToString,
            response_deserializer=v1_dot_users_dot_notification__pb2.NotificationUpdateResponse.FromString,
        )
        self.NotificationDelete = channel.unary_unary(
            "/pro.omni.oms.api.v1.users.notification.NotificationService/NotificationDelete",
            request_serializer=v1_dot_users_dot_notification__pb2.NotificationDeleteRequest.SerializeToString,
            response_deserializer=v1_dot_users_dot_notification__pb2.NotificationDeleteResponse.FromString,
        )
        self.Hidden = channel.unary_unary(
            "/pro.omni.oms.api.v1.users.notification.NotificationService/Hidden",
            request_serializer=v1_dot_users_dot_notification__pb2.HiddenRequest.SerializeToString,
            response_deserializer=v1_dot_users_dot_notification__pb2.HiddenResponse.FromString,
        )


class NotificationServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def NotificationCreate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def NotificationRead(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def NotificationUpdate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def NotificationDelete(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def Hidden(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_NotificationServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "NotificationCreate": grpc.unary_unary_rpc_method_handler(
            servicer.NotificationCreate,
            request_deserializer=v1_dot_users_dot_notification__pb2.NotificationCreateRequest.FromString,
            response_serializer=v1_dot_users_dot_notification__pb2.NotificationCreateResponse.SerializeToString,
        ),
        "NotificationRead": grpc.unary_unary_rpc_method_handler(
            servicer.NotificationRead,
            request_deserializer=v1_dot_users_dot_notification__pb2.NotificationReadRequest.FromString,
            response_serializer=v1_dot_users_dot_notification__pb2.NotificationReadResponse.SerializeToString,
        ),
        "NotificationUpdate": grpc.unary_unary_rpc_method_handler(
            servicer.NotificationUpdate,
            request_deserializer=v1_dot_users_dot_notification__pb2.NotificationUpdateRequest.FromString,
            response_serializer=v1_dot_users_dot_notification__pb2.NotificationUpdateResponse.SerializeToString,
        ),
        "NotificationDelete": grpc.unary_unary_rpc_method_handler(
            servicer.NotificationDelete,
            request_deserializer=v1_dot_users_dot_notification__pb2.NotificationDeleteRequest.FromString,
            response_serializer=v1_dot_users_dot_notification__pb2.NotificationDeleteResponse.SerializeToString,
        ),
        "Hidden": grpc.unary_unary_rpc_method_handler(
            servicer.Hidden,
            request_deserializer=v1_dot_users_dot_notification__pb2.HiddenRequest.FromString,
            response_serializer=v1_dot_users_dot_notification__pb2.HiddenResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "pro.omni.oms.api.v1.users.notification.NotificationService", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class NotificationService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def NotificationCreate(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.users.notification.NotificationService/NotificationCreate",
            v1_dot_users_dot_notification__pb2.NotificationCreateRequest.SerializeToString,
            v1_dot_users_dot_notification__pb2.NotificationCreateResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def NotificationRead(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.users.notification.NotificationService/NotificationRead",
            v1_dot_users_dot_notification__pb2.NotificationReadRequest.SerializeToString,
            v1_dot_users_dot_notification__pb2.NotificationReadResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def NotificationUpdate(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.users.notification.NotificationService/NotificationUpdate",
            v1_dot_users_dot_notification__pb2.NotificationUpdateRequest.SerializeToString,
            v1_dot_users_dot_notification__pb2.NotificationUpdateResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def NotificationDelete(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.users.notification.NotificationService/NotificationDelete",
            v1_dot_users_dot_notification__pb2.NotificationDeleteRequest.SerializeToString,
            v1_dot_users_dot_notification__pb2.NotificationDeleteResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def Hidden(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.users.notification.NotificationService/Hidden",
            v1_dot_users_dot_notification__pb2.HiddenRequest.SerializeToString,
            v1_dot_users_dot_notification__pb2.HiddenResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
from omni.pro.protos.v1.rules import appointment_line_pb2 as v1_dot_rules_dot_appointment__line__pb2


class AppointmentLineServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.AppointmentLineCreate = channel.unary_unary(
            "/pro.omni.oms.api.v1.rules.appointment_line.AppointmentLineService/AppointmentLineCreate",
            request_serializer=v1_dot_rules_dot_appointment__line__pb2.AppointmentLineCreateRequest.SerializeToString,
            response_deserializer=v1_dot_rules_dot_appointment__line__pb2.AppointmentLineCreateResponse.FromString,
        )
        self.AppointmentLineRead = channel.unary_unary(
            "/pro.omni.oms.api.v1.rules.appointment_line.AppointmentLineService/AppointmentLineRead",
            request_serializer=v1_dot_rules_dot_appointment__line__pb2.AppointmentLineReadRequest.SerializeToString,
            response_deserializer=v1_dot_rules_dot_appointment__line__pb2.AppointmentLineReadResponse.FromString,
        )
        self.AppointmentLineUpdate = channel.unary_unary(
            "/pro.omni.oms.api.v1.rules.appointment_line.AppointmentLineService/AppointmentLineUpdate",
            request_serializer=v1_dot_rules_dot_appointment__line__pb2.AppointmentLineUpdateRequest.SerializeToString,
            response_deserializer=v1_dot_rules_dot_appointment__line__pb2.AppointmentLineUpdateResponse.FromString,
        )
        self.AppointmentLineDelete = channel.unary_unary(
            "/pro.omni.oms.api.v1.rules.appointment_line.AppointmentLineService/AppointmentLineDelete",
            request_serializer=v1_dot_rules_dot_appointment__line__pb2.AppointmentLineDeleteRequest.SerializeToString,
            response_deserializer=v1_dot_rules_dot_appointment__line__pb2.AppointmentLineDeleteResponse.FromString,
        )


class AppointmentLineServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def AppointmentLineCreate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def AppointmentLineRead(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def AppointmentLineUpdate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def AppointmentLineDelete(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_AppointmentLineServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "AppointmentLineCreate": grpc.unary_unary_rpc_method_handler(
            servicer.AppointmentLineCreate,
            request_deserializer=v1_dot_rules_dot_appointment__line__pb2.AppointmentLineCreateRequest.FromString,
            response_serializer=v1_dot_rules_dot_appointment__line__pb2.AppointmentLineCreateResponse.SerializeToString,
        ),
        "AppointmentLineRead": grpc.unary_unary_rpc_method_handler(
            servicer.AppointmentLineRead,
            request_deserializer=v1_dot_rules_dot_appointment__line__pb2.AppointmentLineReadRequest.FromString,
            response_serializer=v1_dot_rules_dot_appointment__line__pb2.AppointmentLineReadResponse.SerializeToString,
        ),
        "AppointmentLineUpdate": grpc.unary_unary_rpc_method_handler(
            servicer.AppointmentLineUpdate,
            request_deserializer=v1_dot_rules_dot_appointment__line__pb2.AppointmentLineUpdateRequest.FromString,
            response_serializer=v1_dot_rules_dot_appointment__line__pb2.AppointmentLineUpdateResponse.SerializeToString,
        ),
        "AppointmentLineDelete": grpc.unary_unary_rpc_method_handler(
            servicer.AppointmentLineDelete,
            request_deserializer=v1_dot_rules_dot_appointment__line__pb2.AppointmentLineDeleteRequest.FromString,
            response_serializer=v1_dot_rules_dot_appointment__line__pb2.AppointmentLineDeleteResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "pro.omni.oms.api.v1.rules.appointment_line.AppointmentLineService", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class AppointmentLineService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def AppointmentLineCreate(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.rules.appointment_line.AppointmentLineService/AppointmentLineCreate",
            v1_dot_rules_dot_appointment__line__pb2.AppointmentLineCreateRequest.SerializeToString,
            v1_dot_rules_dot_appointment__line__pb2.AppointmentLineCreateResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def AppointmentLineRead(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.rules.appointment_line.AppointmentLineService/AppointmentLineRead",
            v1_dot_rules_dot_appointment__line__pb2.AppointmentLineReadRequest.SerializeToString,
            v1_dot_rules_dot_appointment__line__pb2.AppointmentLineReadResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def AppointmentLineUpdate(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.rules.appointment_line.AppointmentLineService/AppointmentLineUpdate",
            v1_dot_rules_dot_appointment__line__pb2.AppointmentLineUpdateRequest.SerializeToString,
            v1_dot_rules_dot_appointment__line__pb2.AppointmentLineUpdateResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def AppointmentLineDelete(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.rules.appointment_line.AppointmentLineService/AppointmentLineDelete",
            v1_dot_rules_dot_appointment__line__pb2.AppointmentLineDeleteRequest.SerializeToString,
            v1_dot_rules_dot_appointment__line__pb2.AppointmentLineDeleteResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )



from enum import Enum

from mongoengine import DictField, EnumField, StringField
from omni.pro.models.base import BaseDocument


class StateEnum(Enum):
    error = "error"
    done = "done"
    pending = "pending"


class Logger(BaseDocument):
    state = EnumField(StateEnum)
    model_code = StringField()
    data = DictField()

    meta = {
        "collection": "loggers",
    }



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: v1/rules/appointment_template_line.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from omni.pro.protos.common import base_pb2 as common_dot_base__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n(v1/rules/appointment_template_line.proto\x12\x33pro.omni.oms.api.v1.rules.appointment_template_line\x1a\x11\x63ommon/base.proto\x1a\x1egoogle/protobuf/wrappers.proto"\xf1\x01\n\x17\x41ppointmentTemplateLine\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0b\n\x03\x64\x61y\x18\x02 \x01(\t\x12\x12\n\nhour_start\x18\x03 \x01(\t\x12\x10\n\x08hour_end\x18\x04 \x01(\t\x12\x15\n\rorder_numbers\x18\x05 \x01(\x05\x12*\n\x06\x61\x63tive\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x12\x13\n\x0b\x65xternal_id\x18\x08 \x01(\t\x12?\n\x0cobject_audit\x18\t \x01(\x0b\x32).pro.omni.oms.api.common.base.ObjectAudit"\xbd\x01\n$AppointmentTemplateLineCreateRequest\x12\x0b\n\x03\x64\x61y\x18\x01 \x01(\t\x12\x12\n\nhour_start\x18\x02 \x01(\t\x12\x10\n\x08hour_end\x18\x03 \x01(\t\x12\x15\n\rorder_numbers\x18\x04 \x01(\x05\x12\x13\n\x0b\x65xternal_id\x18\x05 \x01(\t\x12\x36\n\x07\x63ontext\x18\x07 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xe3\x01\n%AppointmentTemplateLineCreateResponse\x12o\n\x19\x61ppointment_template_line\x18\x01 \x01(\x0b\x32L.pro.omni.oms.api.v1.rules.appointment_template_line.AppointmentTemplateLine\x12I\n\x11response_standard\x18\x02 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"\x80\x03\n"AppointmentTemplateLineReadRequest\x12\x37\n\x08group_by\x18\x01 \x03(\x0b\x32%.pro.omni.oms.api.common.base.GroupBy\x12\x35\n\x07sort_by\x18\x02 \x01(\x0b\x32$.pro.omni.oms.api.common.base.SortBy\x12\x34\n\x06\x66ields\x18\x03 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Fields\x12\x34\n\x06\x66ilter\x18\x04 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Filter\x12:\n\tpaginated\x18\x05 \x01(\x0b\x32\'.pro.omni.oms.api.common.base.Paginated\x12\n\n\x02id\x18\x06 \x01(\t\x12\x36\n\x07\x63ontext\x18\x07 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\x9d\x02\n#AppointmentTemplateLineReadResponse\x12p\n\x1a\x61ppointment_template_lines\x18\x01 \x03(\x0b\x32L.pro.omni.oms.api.v1.rules.appointment_template_line.AppointmentTemplateLine\x12\x39\n\tmeta_data\x18\x02 \x01(\x0b\x32&.pro.omni.oms.api.common.base.MetaData\x12I\n\x11response_standard\x18\x03 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"\xcf\x01\n$AppointmentTemplateLineUpdateRequest\x12o\n\x19\x61ppointment_template_line\x18\x01 \x01(\x0b\x32L.pro.omni.oms.api.v1.rules.appointment_template_line.AppointmentTemplateLine\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xe3\x01\n%AppointmentTemplateLineUpdateResponse\x12o\n\x19\x61ppointment_template_line\x18\x01 \x01(\x0b\x32L.pro.omni.oms.api.v1.rules.appointment_template_line.AppointmentTemplateLine\x12I\n\x11response_standard\x18\x02 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"j\n$AppointmentTemplateLineDeleteRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"r\n%AppointmentTemplateLineDeleteResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard2\x86\x07\n\x1e\x41ppointmentTemplateLineService\x12\xd8\x01\n\x1d\x41ppointmentTemplateLineCreate\x12Y.pro.omni.oms.api.v1.rules.appointment_template_line.AppointmentTemplateLineCreateRequest\x1aZ.pro.omni.oms.api.v1.rules.appointment_template_line.AppointmentTemplateLineCreateResponse"\x00\x12\xd2\x01\n\x1b\x41ppointmentTemplateLineRead\x12W.pro.omni.oms.api.v1.rules.appointment_template_line.AppointmentTemplateLineReadRequest\x1aX.pro.omni.oms.api.v1.rules.appointment_template_line.AppointmentTemplateLineReadResponse"\x00\x12\xd8\x01\n\x1d\x41ppointmentTemplateLineUpdate\x12Y.pro.omni.oms.api.v1.rules.appointment_template_line.AppointmentTemplateLineUpdateRequest\x1aZ.pro.omni.oms.api.v1.rules.appointment_template_line.AppointmentTemplateLineUpdateResponse"\x00\x12\xd8\x01\n\x1d\x41ppointmentTemplateLineDelete\x12Y.pro.omni.oms.api.v1.rules.appointment_template_line.AppointmentTemplateLineDeleteRequest\x1aZ.pro.omni.oms.api.v1.rules.appointment_template_line.AppointmentTemplateLineDeleteResponse"\x00\x62\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "v1.rules.appointment_template_line_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    _globals["_APPOINTMENTTEMPLATELINE"]._serialized_start = 149
    _globals["_APPOINTMENTTEMPLATELINE"]._serialized_end = 390
    _globals["_APPOINTMENTTEMPLATELINECREATEREQUEST"]._serialized_start = 393
    _globals["_APPOINTMENTTEMPLATELINECREATEREQUEST"]._serialized_end = 582
    _globals["_APPOINTMENTTEMPLATELINECREATERESPONSE"]._serialized_start = 585
    _globals["_APPOINTMENTTEMPLATELINECREATERESPONSE"]._serialized_end = 812
    _globals["_APPOINTMENTTEMPLATELINEREADREQUEST"]._serialized_start = 815
    _globals["_APPOINTMENTTEMPLATELINEREADREQUEST"]._serialized_end = 1199
    _globals["_APPOINTMENTTEMPLATELINEREADRESPONSE"]._serialized_start = 1202
    _globals["_APPOINTMENTTEMPLATELINEREADRESPONSE"]._serialized_end = 1487
    _globals["_APPOINTMENTTEMPLATELINEUPDATEREQUEST"]._serialized_start = 1490
    _globals["_APPOINTMENTTEMPLATELINEUPDATEREQUEST"]._serialized_end = 1697
    _globals["_APPOINTMENTTEMPLATELINEUPDATERESPONSE"]._serialized_start = 1700
    _globals["_APPOINTMENTTEMPLATELINEUPDATERESPONSE"]._serialized_end = 1927
    _globals["_APPOINTMENTTEMPLATELINEDELETEREQUEST"]._serialized_start = 1929
    _globals["_APPOINTMENTTEMPLATELINEDELETEREQUEST"]._serialized_end = 2035
    _globals["_APPOINTMENTTEMPLATELINEDELETERESPONSE"]._serialized_start = 2037
    _globals["_APPOINTMENTTEMPLATELINEDELETERESPONSE"]._serialized_end = 2151
    _globals["_APPOINTMENTTEMPLATELINESERVICE"]._serialized_start = 2154
    _globals["_APPOINTMENTTEMPLATELINESERVICE"]._serialized_end = 3056
# @@protoc_insertion_point(module_scope)



import csv
import importlib
from ast import literal_eval
from datetime import datetime
from pathlib import Path
from typing import Union

from google.protobuf import json_format
from mongoengine import Document
from omni.pro import redis, util
from omni.pro.config import Config
from omni.pro.database import DatabaseManager, PersistenceTypeEnum, PostgresDatabaseManager
from omni.pro.logger import LoggerTraceback, configure_logger
from omni.pro.microservice import MicroService
from omni.pro.models.base import Audit, Base
from omni.pro.protos.grpc_connector import Event, GRPClient
from omni.pro.protos.v1.users import user_pb2
from omni.pro.protos.v1.utilities.ms_pb2 import Microservice as MicroserviceProto
from omni.pro.stack import ExitStackDocument
from omni.pro.validators import MicroServicePathValidator

logger = configure_logger(name=__name__)


class LoadData(object):
    def __init__(self, base_app: Path, microservice: str, persistence_type: PersistenceTypeEnum):
        self.base_app = base_app
        self.persistence_type = persistence_type
        self.microserivce = microservice

    def get_rpc_manifest_func_class(self):
        return ManifestRPCFunction

    def load_data(self):
        redis_manager = redis.get_redis_manager()
        tenans = redis_manager.get_tenant_codes()
        for tenant in tenans:
            user = redis_manager.get_user_admin(tenant)
            context = {
                "tenant": tenant,
                "user": user.get("id") or "admin",
            }
            micro: MicroserviceProto = self.get_rpc_manifest_func_class()(context).get_micro(
                code=self.microserivce,
            )
            if micro:
                db_params = redis_manager.get_mongodb_config(Config.SERVICE_ID, tenant)
                db_alias = f"{tenant}_{db_params.get('name')}"
                # db_params["db"] = db_params.pop("name")
                # db_manager = DatabaseManager(**db_params)
                db_pg_params = redis_manager.get_postgres_config(Config.SERVICE_ID, tenant)
                self.load_data_micro(db_pg_params, json_format.MessageToDict(micro), context, db_alias)

    def create_user_admin(self, context, rc):
        values = self.load_data_dict(Path(__file__).parent / "data" / "models.user.csv")
        user = UserChannel(context)
        response, success = user.create_users(values)
        if success:
            rc.json().set(
                context.get("tenant"), "$.user_admin", {"id": response.user.id, "username": response.user.username}
            )
        return response, success

    def load_data_dict(self, name_file, mode="r", encoding="utf-8-sig", delimiter=";"):
        try:
            with open(name_file, mode=mode, encoding=encoding) as csv_file:
                reader = csv.DictReader(csv_file, delimiter=delimiter)
                for row in reader:
                    yield row
                return reader
        except FileNotFoundError as e:
            LoggerTraceback.error("File not found exception", e, logger)
        except Exception as e:
            LoggerTraceback.error("An unexpected error has occurred", e, logger)

    def csv_to_class(
        self,
        csv_path: str,
        class_document: Union[Base, Document],
        mode="r",
        encoding="utf-8-sig",
        delimiter: str = ";",
        **kwargs,
    ) -> list[Document]:
        """
        Reads a CSV file and converts each row to a document.
        """
        docs_to_insert = [
            class_document(**dict(row) | kwargs)
            for row in self.load_data_dict(csv_path, mode=mode, encoding=encoding, delimiter=delimiter)
        ]
        return docs_to_insert

    def load_data_micro(self, db_pg_params: dict, micro: dict, context: dict, db_alias):
        tenant = context.get("tenant")
        for idx, file in enumerate(micro.get("data") or []):
            if file.get("load"):
                continue
            file_path = self.base_app / file.get("path")
            models, file_py, model_str = file.get("path").split("/")[1].split(".")[:-1]
            model_str = util.to_camel_case(model_str)
            modulo = importlib.import_module(f"{models}.{file_py}")
            if not hasattr(modulo, model_str):
                logger.error(f"Class not found {model_str} in {modulo}")
                continue
            attr_data = [{f"set__data__{idx}__load": True}]
            logger.info(f"Load data {micro.get('code')} - {tenant} - {file.get('path')}")
            if self.persistence_type == PersistenceTypeEnum.NO_SQL:
                self.load_data_document(context, file_path, db_alias, modulo, model_str)
            elif self.persistence_type == PersistenceTypeEnum.SQL:
                self.load_data_model(db_pg_params, context, file_path, modulo, model_str)
            self.get_rpc_manifest_func_class()(context).update_micro(
                {"microservice": {"id": micro.get("id"), "settings": micro.get("settings"), "data": attr_data}}
            )

    def load_data_document(self, context, file_path, db_alias, modulo, model_str):
        with ExitStackDocument(
            (doc_class := getattr(modulo, model_str)).reference_list(),
            db_alias=db_alias,
            use_doc_classes=True,
        ):
            audit = Audit(created_by=context.get("user"))
            audit.updated_by = context.get("user")
            audit.updated_at = datetime.utcnow()
            docs_to_insert = self.csv_to_class(file_path, doc_class, context=context, audit=audit)
            doc_class.objects.insert(docs_to_insert, load_bulk=False)

    def load_data_model(self, db_pg_params: dict, context: dict, file_path: str, modulo, model_str: str):
        with PostgresDatabaseManager(**db_pg_params) as session:
            audit = dict(
                created_by=context.get("user"),
                updated_by=context.get("user"),
                created_at=datetime.now(),
                updated_at=datetime.now(),
                tenant=context.get("tenant"),
            )
            model_class = getattr(modulo, model_str)
            models_to_insert = self.csv_to_class(file_path, model_class, **audit)
            session.bulk_save_objects(models_to_insert)


class Manifest(object):
    def __init__(self, base_app: Path):
        self.base_app = base_app

    def get_rpc_manifest_func_class(self):
        return ManifestRPCFunction

    def get_manifest(self):
        file_name = self.base_app / "__manifest__.py"
        if not file_name.exists():
            logger.warning(f"Manifest file not found {file_name}")
            return {}
        with open(file_name, "r") as f:
            data = f.read()
        manifest = literal_eval(data)
        return manifest

    def validate_manifest(
        self, context: dict, manifest: dict = {}, micro_data: [dict] = [], micro_settings: [dict] = []
    ):
        manifest = manifest or self.get_manifest()
        data_validated = MicroServicePathValidator(self.base_app).load(
            manifest | {"context": context}, micro_data=micro_data, micro_settings=micro_settings
        )
        return data_validated

    def load_manifest(self):
        manifest = self.get_manifest()
        if not manifest:
            return
        redis_manager = redis.get_redis_manager()
        tenans = redis_manager.get_tenant_codes()
        for tenant in tenans:
            user = redis_manager.get_user_admin(tenant)
            context = {
                "tenant": tenant,
                "user": user.get("id") or "admin",
            }
            rpc_func: ManifestRPCFunction = self.get_rpc_manifest_func_class()(context)
            try:
                micro = rpc_func.get_micro(manifest.get("code"))
                manifest_data = self.validate_manifest(
                    context=context,
                    manifest=manifest,
                    micro_data=json_format.MessageToDict(micro.data),
                    micro_settings=json_format.MessageToDict(micro.settings),
                )
                rpc_func.load_manifest(manifest_data)
            except Exception as e:
                LoggerTraceback.error("Load manifest exception", e, logger)


class ManifestRPCFunction(object):
    def __init__(self, context: dict) -> None:
        self.context = context
        self.service_id = MicroService.SAAS_MS_UTILITIES.value
        self.microservice_module_grpc = "v1.utilities.ms_pb2_grpc"
        self.microservice_stub_classname = "MicroserviceServiceStub"
        self.microservice_module_pb2 = "v1.utilities.ms_pb2"

        self.event = Event(
            module_grpc=self.microservice_module_grpc,
            stub_classname=self.microservice_stub_classname,
            rpc_method="",
            module_pb2=self.microservice_module_pb2,
            request_class="",
            params={},
        )

    def load_manifest(self, params):
        self.event.update(
            rpc_method="MicroserviceCreate",
            request_class="MicroserviceCreateRequest",
            params=params | {"context": self.context},
        )
        response, success = GRPClient(service_id=self.service_id).call_rpc_fuction(self.event)
        logger.info(f"Load manifest {response.microservice.code} - status: {success}")
        return response

    def get_micro(self, code: str) -> MicroserviceProto:
        self.event.update(
            rpc_method="MicroserviceRead",
            request_class="MicroserviceReadRequest",
            params={"filter": {"filter": f"[('code','=','{code}')]"}} | {"context": self.context},
        )
        response, _s = GRPClient(service_id=self.service_id).call_rpc_fuction(self.event)
        micro: MicroserviceProto = response.microservices[0] if response.microservices else MicroserviceProto()
        return micro

    def update_micro(self, params: dict):
        self.event.update(
            rpc_method="MicroserviceUpdate",
            request_class="MicroserviceUpdateRequest",
            params=params | {"context": self.context},
        )
        response, success = GRPClient(service_id=self.service_id).call_rpc_fuction(self.event)
        logger.info(f"Update manifest {response.microservice.code} - status: {success}")
        return response


class UserChannel(object):
    def __init__(self, context: dict) -> None:
        self.context = context
        self.service_id = MicroService.SAAS_MS_USER.value
        self.user_module_grpc = "v1.users.user_pb2_grpc"
        self.user_stub_classname = "UsersServiceStub"
        self.user_module_pb2 = "v1.users.user_pb2"

        self.event = Event(
            module_grpc=self.user_module_grpc,
            stub_classname=self.user_stub_classname,
            rpc_method="",
            module_pb2=self.user_module_pb2,
            request_class="",
            params={},
        )

    def create_users(self, list_value):
        response = user_pb2.UserCreateResponse(), False
        for value in list_value:
            self.context["user"] = self.context.get("user") or value.get("sub")
            self.event.update(
                rpc_method="UserCreate",
                request_class="UserCreateRequest",
                params={
                    "context": self.context,
                    "email": value.get("email"),
                    "email_confirm": value.get("email"),
                    "language": {"code": "01", "code_name": "CO"},
                    "name": value.get("name"),
                    "password": value.get("password"),
                    "password_confirm": value.get("password"),
                    "timezone": {"code": "01", "code_name": "CO"},
                    "username": value.get("username"),
                    "is_superuser": True,
                },
            )
            response = GRPClient(service_id=self.service_id).call_rpc_fuction(self.event)
        return response



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: v1/users/notification.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from omni.pro.protos.common import base_pb2 as common_dot_base__pb2
from omni.pro.protos.v1.users import user_pb2 as v1_dot_users_dot_user__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x1bv1/users/notification.proto\x12&pro.omni.oms.api.v1.users.notification\x1a\x13v1/users/user.proto\x1a\x1cgoogle/protobuf/struct.proto\x1a\x11\x63ommon/base.proto\x1a\x1egoogle/protobuf/wrappers.proto"\xbd\x02\n\x0cNotification\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x32\n\x04user\x18\x03 \x01(\x0b\x32$.pro.omni.oms.api.v1.users.user.User\x12\x0c\n\x04type\x18\x04 \x01(\t\x12(\n\x04show\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x12*\n\x06\x61\x63tive\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x12%\n\x04\x64\x61ta\x18\x07 \x01(\x0b\x32\x17.google.protobuf.Struct\x12\x13\n\x0b\x65xternal_id\x18\x08 \x01(\t\x12?\n\x0cobject_audit\x18\t \x01(\x0b\x32).pro.omni.oms.api.common.base.ObjectAudit"\x92\x02\n\x19NotificationCreateRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07user_id\x18\x02 \x01(\t\x12\x0c\n\x04type\x18\x03 \x01(\t\x12(\n\x04show\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x12*\n\x06\x61\x63tive\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x12%\n\x04\x64\x61ta\x18\x06 \x01(\x0b\x32\x17.google.protobuf.Struct\x12\x13\n\x0b\x65xternal_id\x18\x07 \x01(\t\x12\x36\n\x07\x63ontext\x18\x08 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xb3\x01\n\x1aNotificationCreateResponse\x12J\n\x0cnotification\x18\x01 \x01(\x0b\x32\x34.pro.omni.oms.api.v1.users.notification.Notification\x12I\n\x11response_standard\x18\x02 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"\xf5\x02\n\x17NotificationReadRequest\x12\x37\n\x08group_by\x18\x01 \x03(\x0b\x32%.pro.omni.oms.api.common.base.GroupBy\x12\x35\n\x07sort_by\x18\x02 \x01(\x0b\x32$.pro.omni.oms.api.common.base.SortBy\x12\x34\n\x06\x66ields\x18\x03 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Fields\x12\x34\n\x06\x66ilter\x18\x04 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Filter\x12:\n\tpaginated\x18\x05 \x01(\x0b\x32\'.pro.omni.oms.api.common.base.Paginated\x12\n\n\x02id\x18\x06 \x01(\t\x12\x36\n\x07\x63ontext\x18\x07 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xed\x01\n\x18NotificationReadResponse\x12K\n\rnotifications\x18\x01 \x03(\x0b\x32\x34.pro.omni.oms.api.v1.users.notification.Notification\x12\x39\n\tmeta_data\x18\x02 \x01(\x0b\x32&.pro.omni.oms.api.common.base.MetaData\x12I\n\x11response_standard\x18\x03 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"\x9f\x01\n\x19NotificationUpdateRequest\x12J\n\x0cnotification\x18\x01 \x01(\x0b\x32\x34.pro.omni.oms.api.v1.users.notification.Notification\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xb3\x01\n\x1aNotificationUpdateResponse\x12J\n\x0cnotification\x18\x01 \x01(\x0b\x32\x34.pro.omni.oms.api.v1.users.notification.Notification\x12I\n\x11response_standard\x18\x02 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"_\n\x19NotificationDeleteRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"g\n\x1aNotificationDeleteResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"a\n\rHiddenRequest\x12\x18\n\x10notification_ids\x18\x01 \x03(\t\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"[\n\x0eHiddenResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard2\x8a\x06\n\x13NotificationService\x12\x9d\x01\n\x12NotificationCreate\x12\x41.pro.omni.oms.api.v1.users.notification.NotificationCreateRequest\x1a\x42.pro.omni.oms.api.v1.users.notification.NotificationCreateResponse"\x00\x12\x97\x01\n\x10NotificationRead\x12?.pro.omni.oms.api.v1.users.notification.NotificationReadRequest\x1a@.pro.omni.oms.api.v1.users.notification.NotificationReadResponse"\x00\x12\x9d\x01\n\x12NotificationUpdate\x12\x41.pro.omni.oms.api.v1.users.notification.NotificationUpdateRequest\x1a\x42.pro.omni.oms.api.v1.users.notification.NotificationUpdateResponse"\x00\x12\x9d\x01\n\x12NotificationDelete\x12\x41.pro.omni.oms.api.v1.users.notification.NotificationDeleteRequest\x1a\x42.pro.omni.oms.api.v1.users.notification.NotificationDeleteResponse"\x00\x12y\n\x06Hidden\x12\x35.pro.omni.oms.api.v1.users.notification.HiddenRequest\x1a\x36.pro.omni.oms.api.v1.users.notification.HiddenResponse"\x00\x62\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "v1.users.notification_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    _globals["_NOTIFICATION"]._serialized_start = 174
    _globals["_NOTIFICATION"]._serialized_end = 491
    _globals["_NOTIFICATIONCREATEREQUEST"]._serialized_start = 494
    _globals["_NOTIFICATIONCREATEREQUEST"]._serialized_end = 768
    _globals["_NOTIFICATIONCREATERESPONSE"]._serialized_start = 771
    _globals["_NOTIFICATIONCREATERESPONSE"]._serialized_end = 950
    _globals["_NOTIFICATIONREADREQUEST"]._serialized_start = 953
    _globals["_NOTIFICATIONREADREQUEST"]._serialized_end = 1326
    _globals["_NOTIFICATIONREADRESPONSE"]._serialized_start = 1329
    _globals["_NOTIFICATIONREADRESPONSE"]._serialized_end = 1566
    _globals["_NOTIFICATIONUPDATEREQUEST"]._serialized_start = 1569
    _globals["_NOTIFICATIONUPDATEREQUEST"]._serialized_end = 1728
    _globals["_NOTIFICATIONUPDATERESPONSE"]._serialized_start = 1731
    _globals["_NOTIFICATIONUPDATERESPONSE"]._serialized_end = 1910
    _globals["_NOTIFICATIONDELETEREQUEST"]._serialized_start = 1912
    _globals["_NOTIFICATIONDELETEREQUEST"]._serialized_end = 2007
    _globals["_NOTIFICATIONDELETERESPONSE"]._serialized_start = 2009
    _globals["_NOTIFICATIONDELETERESPONSE"]._serialized_end = 2112
    _globals["_HIDDENREQUEST"]._serialized_start = 2114
    _globals["_HIDDENREQUEST"]._serialized_end = 2211
    _globals["_HIDDENRESPONSE"]._serialized_start = 2213
    _globals["_HIDDENRESPONSE"]._serialized_end = 2304
    _globals["_NOTIFICATIONSERVICE"]._serialized_start = 2307
    _globals["_NOTIFICATIONSERVICE"]._serialized_end = 3085
# @@protoc_insertion_point(module_scope)
